package mc600.boardgames;

import mc600.gamemanager.*;

public class ConnectBoard {
    protected int xdim;
    protected int ydim;
    protected int[][] quadrados;
    protected boolean gameIsOver;
    protected int victory;
    protected int nLances;
    
    /**
     * Insert the method's description here.
     * Creation date: (3/8/00 7:53:14 PM)
     * @param xdim int
     * @param ydim int
     */
    public ConnectBoard(int xdim, int ydim) throws Exception {

	// seta dimensoes do tabuleiro 
	setXdim(xdim);
	setYdim(ydim);

	// cria quadrados, que sao a composicao basica do tabuleiro
	quadrados = new int[getXdim()][getYdim()];
	
	for (int i=0; i < getXdim(); i++)
	    for (int j=0; j < getYdim(); j++)
		quadrados[i][j] = 0;

	// inicializa condicoes de jogo
	gameIsOver = false;
	victory = 0;
	nLances = 0;
    }

    public int getXdim() {
	return xdim;
    }

    public int getYdim() {
	return ydim;
    }

    public boolean gameOver() {
	return gameIsOver;
    }

    public boolean victoryP1() {
	return victory == 1;
    }

    public boolean victoryP2() {
	return victory == 2;
    }
    
    /**
     * Retorna true caso o jogador tenha direito a mais um lance.
     * (no caso de Connect Four, sempre retorna false).
     *
     * O tabuleiro e' formado por uma array bidimensional contendo
     * 0, 1 ou 2, para indicar casa vazia, ocupada por P1 ou ocupada
     * por P2, respectivamente.
     *
     * Depois de cada lance e' feita a verificacao para saber
     * se o jogo acabou, ou seja, se o jogador fechou uma linha
     * com pelo menos 4 casas.
     *
     * Creation date: (3/24/00 3:27:06 AM)
     * @return boolean
     * @param move java.lang.String
     */
    public boolean play(String move, boolean isP1)
	throws InvalidMoveException {

	Square casa = new Square(move);
	
	//verifica se cordenadas estao dentro do tabuleiro
	if ( ! dentro(casa) )

	    throw new InvalidMoveException("Lance fora do tabuleiro <" 
					   + move + "> ");

	//verifica se casa esta' vazia e a de baixo ocupada (caso exista)
	if ( occupied(casa) || ( dentro(casa.abaixo()) &&
				 empty(casa.abaixo()) ) )

	    throw new InvalidMoveException("Lance invalido <" 
					   + move + "> ");
	
	// marca lance
	int vez = isP1 ? 1 : 2;
	marca(casa,vez);
	nLances++;

	// ve^ se fechou
	if (ganhou(vez,casa)) {
	    gameIsOver = true;
	    victory = vez;
	}

	// ve^ se nao tem mais lugar
	if (nLances == xdim*ydim)
	    gameIsOver = true;
	
	return false;
    }

    private void setXdim(int newXdim) throws Exception{

	if (newXdim < 2)
	    throw new Exception("Dimensao do tabuleiro invalida.\n");
		
	xdim = newXdim;
    }

    private void setYdim(int newYdim) throws Exception{

	if (newYdim < 2)
	    throw new Exception("Dimensao do tabuleiro invalida.\n");
	
	ydim = newYdim;
    }

    public boolean dentro(Square casa) {
	return
	    (casa.getX() < xdim) && (casa.getX() >= 0) && 
	    (casa.getY() < ydim) && (casa.getY() >= 0);
    }

    public int conteudo(Square casa) {
	return quadrados[casa.getX()][casa.getY()];
    }

    public boolean empty(Square casa) {
	return conteudo(casa) == 0;
    }

    public boolean occupied(Square casa) {
	return ! empty(casa);
    }

    public void marca (Square casa, int vez) {
	quadrados[casa.getX()][casa.getY()] = vez;
    }
    
    // verifica se jogador "vez" ganhou jogando en (x,y)
    public boolean ganhou(int vez, Square casa) {
	return ( (4 <= qtasConsecutivas(vez,casa,new Direction(1,0))) ||
		 (4 <= qtasConsecutivas(vez,casa,new Direction(0,1))) ||
		 (4 <= qtasConsecutivas(vez,casa,new Direction(1,1))) ||
		 (4 <= qtasConsecutivas(vez,casa,new Direction(1,-1))) );
    }

    public int qtasConsecutivas(int vez, Square casa, Direction d) {
	return 1
	    + qtasDireto(vez,casa.seguinte(d),d)
	    + qtasDireto(vez,casa.seguinte(d.oposto()),d.oposto());
    }

    public int qtasDireto(int vez, Square casa, Direction d) {
	if (!dentro(casa))
	    return 0;
	else if (conteudo(casa) == vez)
	    return 1 + qtasDireto(vez,casa.seguinte(d),d);
        else
	    return 0;
    }
}
